package com.yonge.cooleshow.cms.controller;

import com.yonge.cooleshow.cms.dal.entity.NewsStatusEnum;
import com.yonge.cooleshow.cms.dto.AppHomeDto;
import com.yonge.cooleshow.cms.dto.SysNewsInformationDto;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.cms.controller.queryinfo.NewsInformationQueryInfo;
import com.yonge.cooleshow.cms.dal.dao.StudentRegistrationDao;
import com.yonge.cooleshow.cms.dal.dao.SysNewsInformationDao;
import com.yonge.cooleshow.cms.dal.entity.SysNewsInformation;
import com.yonge.cooleshow.cms.service.SysNewsInformationService;
import com.yonge.cooleshow.cms.service.SysNewsTypeService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.toolset.log.model.AuditLogAnnotation;
import com.yonge.toolset.utils.collection.MapUtil;

import javax.validation.Valid;

@RestController
@RequestMapping("news")
@Api(tags = "资讯/广告/闪页/轮播图服务")
public class NewsController extends BaseController {

	@Autowired
	private SysNewsInformationService sysNewsInformationService;

	@Autowired
	private SysUserFeignService sysUserFeignService;

	@ApiOperation(value = "资讯/广告/闪页/轮播图 列表分页查询", httpMethod="POST", consumes="application/json", produces="application/json")
	@PostMapping(value = "/page",  consumes="application/json", produces="application/json")
	public HttpResponseResult<PageInfo<SysNewsInformationDto>> getList(@Valid @RequestBody NewsInformationQueryInfo queryInfo) {
		
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		return succeed(sysNewsInformationService.selectPage(queryInfo));
	}


	@ApiOperation("查询资讯/广告/闪页/轮播图 详情")
	@ApiImplicitParam(name = "id", value = "资讯ID编号", required = true, dataType = "Long", paramType = "path")
	@GetMapping(value = "/query/{id}")
	public HttpResponseResult<SysNewsInformationDto> query(@PathVariable Long id) {

		return succeed(sysNewsInformationService.queryById(id));
	}

	@ApiOperation(value = "新增资讯/广告/闪页/轮播图 ", httpMethod="POST", consumes="application/json", produces="application/json")
	@PostMapping(value = "/add",  consumes="application/json", produces="application/json")
	@AuditLogAnnotation(operateName = "资讯新增",interfaceURL = "news/add")
	// @PreAuthorize("@pcs.hasPermissions('news/add')")
	public Object add(@Valid @RequestBody  SysNewsInformation newsInfo) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null || sysUser.getId() == null) {
			return failed("用户信息获取失败");
		}
		newsInfo.setStatus(NewsStatusEnum.HIDDEN);
		newsInfo.setCreateTime(new Date());
		newsInfo.setCreateBy(sysUser.getId());
		newsInfo.setUpdateTime(new Date());
		newsInfo.setUpdateBy(sysUser.getId());
		return succeed(sysNewsInformationService.insert(newsInfo));
	}

	@ApiOperation(value = "更新资讯/广告/闪页/轮播图 ", httpMethod="POST", consumes="application/json", produces="application/json")
	@PostMapping(value = "/update",  consumes="application/json", produces="application/json")
	@AuditLogAnnotation(operateName = "资讯更新",interfaceURL = "news/update")
	// @PreAuthorize("@pcs.hasPermissions('news/update')")
	public Object update(@Valid @RequestBody  SysNewsInformation newsInfo) {
		SysUser sysUser = sysUserFeignService.queryUserInfo();
		if (sysUser == null || sysUser.getId() == null) {
			return failed("用户信息获取失败");
		}

		SysNewsInformationDto sysNewsInformationDto = sysNewsInformationService.queryById(newsInfo.getId());
		if (sysNewsInformationDto == null) {
			return failed("未找到相关信息");
		}
		if(NewsStatusEnum.SHOW.getCode().equals(sysNewsInformationDto.getStatus().getCode())) {
			return failed("启用状态下，不可修改");
		}
		newsInfo.setUpdateTime(new Date());
		newsInfo.setUpdateBy(sysUser.getId());

		return succeed(sysNewsInformationService.update(newsInfo));
	}

	@ApiOperation("删除")
	@PostMapping(value = "/del/{id}")
	@AuditLogAnnotation(operateName = "资讯删除",interfaceURL = "news/del")
	// @PreAuthorize("@pcs.hasPermissions('news/del')")
	public Object add(@PathVariable("id") Long id) {
		SysNewsInformationDto sysNewsInformationDto = sysNewsInformationService.queryById(id);
		if (sysNewsInformationDto == null) {
			return failed("未找到相关信息");
		}
		if(NewsStatusEnum.SHOW.getCode().equals(sysNewsInformationDto.getStatus().getCode())) {
			return failed("启用状态下，不可删除");
		}
		return succeed(sysNewsInformationService.deleteWithLogical(id));
	}

	@ApiOperation("启用/停用")
	@PostMapping(value = "/status/{id}")
	@AuditLogAnnotation(operateName = "资讯启用/停用",interfaceURL = "news/update")
	// @PreAuthorize("@pcs.hasPermissions('news/del')")
	public Object status(@PathVariable("id") Long id) {
		SysNewsInformationDto sysNewsInformationDto = sysNewsInformationService.queryById(id);
		if (sysNewsInformationDto == null) {
			return failed("未找到相关信息");
		}
		return succeed(sysNewsInformationService.updateStatus(id));
	}

	@ApiOperation("app首页banner、按钮、资讯")
	@PostMapping(value = "/app/home")
	public HttpResponseResult<AppHomeDto> appHome() {
		AppHomeDto appHomeDto = new AppHomeDto();
		NewsInformationQueryInfo queryInfo = new NewsInformationQueryInfo();
		queryInfo.setStatus(NewsStatusEnum.SHOW);
		queryInfo.setPlatformType("STUDENT");
		// 1热门资讯
		queryInfo.setType(1);
		queryInfo.setPage(1);
		queryInfo.setRows(3);
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		List<SysNewsInformationDto> rows = sysNewsInformationService.selectPage(queryInfo).getRows();
		appHomeDto.setInformation(rows);

		// 轮播图
		queryInfo.setType(4);
		queryInfo.setRows(999);
		params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		appHomeDto.setBanner(sysNewsInformationService.selectPage(queryInfo).getRows());
		// 闪页
		queryInfo.setType(3);
		queryInfo.setRows(999);
		params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		appHomeDto.setFlashPage(sysNewsInformationService.selectPage(queryInfo).getRows());
		return succeed(appHomeDto);
	}
}
