package com.yonge.cooleshow.admin.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.entity.MemberPriceSettings;
import com.yonge.cooleshow.biz.dal.service.MemberPriceSettingsService;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.page.PageInfo;
import com.yonge.cooleshow.common.page.QueryInfo;

@RequestMapping("memberPriceSettings")
@Api(tags = "会员价格设置")
@RestController
public class MemberPriceSettingsController extends BaseController {

	@Autowired
	private MemberPriceSettingsService memberPriceSettingsService;

	@Autowired
	private SysUserFeignService sysUserFeignService;

	@ApiOperation("分页查询")
	@GetMapping(value = "/list")
	@PreAuthorize("@pcs.hasPermissions('memberPriceSettings/list')")
	public HttpResponseResult<PageInfo<MemberPriceSettings>> getList(QueryInfo queryInfo) {
		return succeed(memberPriceSettingsService.queryPage(queryInfo));
	}

	@ApiOperation("更新")
	@PostMapping(value = "/update", consumes = MediaType.APPLICATION_JSON_UTF8_VALUE, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
	@PreAuthorize("@pcs.hasPermissions('memberPriceSettings/update')")
	public HttpResponseResult<Object> update(@RequestBody MemberPriceSettings memberPriceSettings) {
		SysUser user = sysUserFeignService.queryUserInfo();

		if (user == null || user.getId() == null) {
			return failed(HttpStatus.FORBIDDEN, "请登录");
		}
		memberPriceSettings.setUpdateBy(user.getId()).setUpdateTime(new Date());
		memberPriceSettingsService.update(memberPriceSettings);
		return succeed();
	}
}
