package com.yonge.cooleshow.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;

import com.yonge.cooleshow.biz.dal.support.PageUtil;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.page.PageInfo;
import com.yonge.toolset.utils.string.StringUtil;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import com.yonge.cooleshow.biz.dal.vo.UserBankCardVo;
import com.yonge.cooleshow.biz.dal.dto.search.UserBankCardSearch;
import com.yonge.cooleshow.biz.dal.entity.UserBankCard;
import com.yonge.cooleshow.biz.dal.service.UserBankCardService;

@RestController
@RequestMapping("/userBankCard")
@Api(value = "用户绑卡记录表", tags = "用户绑卡记录表")
public class UserBankCardController extends BaseController {

    @Autowired
    private UserBankCardService userBankCardService;

	/**
     * 查询单条
     */
    @GetMapping("/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入id")
    public HttpResponseResult<UserBankCardVo> detail(@PathVariable("id") Long id) {
    	return succeed(userBankCardService.detail(id));
	}
    
    /**
     * 查询分页
     */
    @PostMapping("/page")
    @ApiOperation(value = "查询分页", notes = "传入userBankCardSearch")
    public HttpResponseResult<PageInfo<UserBankCardVo>> page(@RequestBody UserBankCardSearch query) {
		IPage<UserBankCardVo> pages = userBankCardService.selectPage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
	}
    
    /**
	 * 新增
	 */
	@PostMapping("/save")
	@ApiOperation(value = "新增", notes = "传入userBankCard")
	public HttpResponseResult save(@Valid @RequestBody UserBankCard userBankCard) {
    	return status(userBankCardService.save(userBankCard));
	}
    
    /**
	 * 修改
	 */
	@PostMapping("/update")
	@ApiOperation(value = "修改", notes = "传入userBankCard")
	public HttpResponseResult update(@Valid @RequestBody UserBankCard userBankCard) {
        return status(userBankCardService.updateById(userBankCard));
	}
    
    /**
	 * 新增或修改
	 */
    @PostMapping("/submit")
    @ApiOperation(value = "新增或修改", notes = "传入userBankCard")
	public HttpResponseResult submit(@Valid @RequestBody UserBankCard userBankCard) {
        return status(userBankCardService.saveOrUpdate(userBankCard));
    }

 	/**
	 * 删除
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public HttpResponseResult remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        if (StringUtil.isEmpty(ids)) {
			return failed("参数不能为空");
		}
		return status(userBankCardService.removeByIds(StringUtil.toLongList(ids)));
	}
}