package com.yonge.cooleshow.admin.controller;

import com.yonge.cooleshow.auth.api.client.SysUserFeignService;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;

import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import com.yonge.cooleshow.biz.dal.vo.VipCardVo;
import com.yonge.cooleshow.biz.dal.entity.VipCard;
import com.yonge.cooleshow.biz.dal.service.VipCardService;

import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/vipCard")
@Api(value = "会员卡表", tags = "会员卡表")
public class VipCardController extends BaseController {

    @Autowired
    private VipCardService vipCardService;
    @Autowired
    private SysUserFeignService sysUserFeignService;

	/**
     * 查询单条
     */
    @GetMapping("/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入id")
    @PreAuthorize("@pcs.hasPermissions('vipCard/detail')")
    public HttpResponseResult<VipCardVo> detail(@PathVariable("id") Long id) {
    	return succeed(vipCardService.detail(id));
	}
    
    /**
     * 查询分页
     */
    @PostMapping("/list")
    @ApiOperation(value = "查询列表")
    @PreAuthorize("@pcs.hasPermissions('vipCard/list')")
    public HttpResponseResult<List<VipCardVo>> list() {
		List<VipCardVo> list = vipCardService.selectList();
        return succeed(list);
	}
    
    /**
	 * 修改
	 */
	@PostMapping("/update")
	@ApiOperation(value = "修改", notes = "传入vipCard")
    @PreAuthorize("@pcs.hasPermissions('vipCard/update')")
    public HttpResponseResult update(@Valid @RequestBody VipCard vipCard) {
        SysUser user = sysUserFeignService.queryUserInfo();
        if (user == null || null == user.getId()) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        vipCard.setUpdateBy(user.getId());
        vipCard.setUpdateTime(new Date());
        return status(vipCardService.updateById(vipCard));
	}
}