package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.biz.dal.dto.req.OrderReq;
import com.yonge.cooleshow.biz.dal.entity.VipCardRecord;
import com.yonge.cooleshow.biz.dal.enums.TimeTypeEnum;
import com.yonge.cooleshow.biz.dal.service.StudentService;
import com.yonge.cooleshow.biz.dal.service.VipCardRecordService;
import com.yonge.cooleshow.biz.dal.vo.StudentVo;
import com.yonge.cooleshow.biz.dal.vo.UserOrderDetailVo;
import com.yonge.cooleshow.biz.dal.vo.res.OrderCreateRes;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.yonge.cooleshow.biz.dal.entity.VipCard;
import com.yonge.cooleshow.biz.dal.vo.VipCardVo;
import com.yonge.cooleshow.biz.dal.dao.VipCardDao;
import com.yonge.cooleshow.biz.dal.service.VipCardService;

import java.util.Calendar;
import java.util.Date;
import java.util.List;


@Service
public class VipCardServiceImpl extends ServiceImpl<VipCardDao, VipCard> implements VipCardService {
    private final static Logger log = LoggerFactory.getLogger(VipCardServiceImpl.class);
    @Autowired
    private VipCardRecordService vipCardRecordService;
    @Autowired
    private StudentService studentService;

    @Override
    public VipCardVo detail(Long id) {
        VipCardVo detail = baseMapper.detail(id);
        detail.setTimeTypeName(detail.getTimeType().name());
        return detail;
    }

    @Override
    public List<VipCardVo> selectList() {
        List<VipCardVo> vipCardVos = baseMapper.selectList();
        vipCardVos.forEach(o -> o.setTimeTypeName(o.getTimeType().name()));
        return vipCardVos;
    }

    @Override
    public HttpResponseResult<OrderCreateRes> orderCreate(OrderReq.OrderReqInfo orderReqInfo) {
        VipCardVo detail = detail(Long.parseLong(orderReqInfo.getBizContent().toString()));
        if (null == detail) {
            return HttpResponseResult.failed("未找到会员卡信息");
        }
        //插入会员卡信息到会员卡记录表
        OrderCreateRes orderCreateRes = new OrderCreateRes();
        orderCreateRes.setRes(true);
        orderCreateRes.setBizId(detail.getId());
        orderCreateRes.setBizContent("会员卡购买-" + detail.getTimeTypeName());
        orderCreateRes.setGoodNum(1);
        orderCreateRes.setOriginalPrice(detail.getOriginalPrice());
        orderCreateRes.setExpectPrice(detail.getActualPrice());
        return HttpResponseResult.succeed(orderCreateRes);
    }


    @Override
    public void orderSuccess(UserOrderDetailVo orderDetailVo) {
        VipCardVo detail = detail(orderDetailVo.getBizId());
        if (null == detail) {
            return;
        }
        StudentVo studentVo = studentService.detail(orderDetailVo.getUserId());
        //修改用户会员时长
        Date membershipEndTime = studentVo.getMembershipEndTime();
        Calendar now = Calendar.getInstance();

        VipCardRecord vipCardRecord = new VipCardRecord();
        vipCardRecord.setUserId(orderDetailVo.getUserId());
        vipCardRecord.setOrderId(orderDetailVo.getOrderId());
        vipCardRecord.setVipCardId(detail.getId());


        if (null == membershipEndTime || membershipEndTime.before(new Date())) {
            //没有会员、会员已过期
            studentVo.setMembershipStartTime(now.getTime());
        } else {
            now.setTime(studentVo.getMembershipEndTime());
        }
        vipCardRecord.setStartTime(now.getTime());
        if (TimeTypeEnum.MONTH.equals(detail.getTimeType())) {
            now.add(Calendar.MONTH, 1);
        } else if (TimeTypeEnum.QUARTER.equals(detail.getTimeType())) {
            now.add(Calendar.MONTH, 3);
        } else if (TimeTypeEnum.YEAR.equals(detail.getTimeType())) {
            now.add(Calendar.YEAR, 1);
        }
        vipCardRecord.setEndTime(now.getTime());
        studentVo.setMembershipEndTime(now.getTime());
        studentVo.setMemberRankSettingId(1l);
        studentService.updateById(studentVo);
        //会员购买记录入库
        vipCardRecordService.save(vipCardRecord);
    }


}